@@ -62,6 +62,9 @@ from pathlib import Path
 from packaging.version import Version as V
 import unicodedata
 
+from cryptography import x509
+from cryptography.hazmat.backends import default_backend
+
 from nbxmpp.namespaces import Namespace
 from nbxmpp.const import Role
 from nbxmpp.const import Chatstate
@@ -75,9 +78,6 @@ from nbxmpp.protocol import JID
 from nbxmpp.protocol import InvalidJid
 from nbxmpp.protocol import Iq
 
-from OpenSSL.crypto import X509, load_certificate
-from OpenSSL.crypto import FILETYPE_PEM
-
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
@@ -87,6 +87,7 @@ import precis_i18n.codec  # noqa: F401
 
 from gajim.common import app
 from gajim.common import configpaths
+from gajim.common import iana
 from gajim.common.i18n import Q_
 from gajim.common.i18n import _
 from gajim.common.i18n import ngettext
@@ -94,13 +95,12 @@ from gajim.common.i18n import get_rfc5646_lang
 from gajim.common.const import SHOW_STRING
 from gajim.common.const import SHOW_STRING_MNEMONIC
 from gajim.common.const import URIType
-from gajim.common.const import URIAction
+from gajim.common.const import XmppUriQuery
 from gajim.common.const import GIO_TLS_ERRORS
 from gajim.common.const import SHOW_LIST
 from gajim.common.const import CONSONANTS
 from gajim.common.const import VOWELS
 from gajim.common.regex import INVALID_XML_CHARS_REGEX
-from gajim.common.regex import STH_AT_STH_DOT_STH_REGEX
 from gajim.common.structs import URI
 from gajim.common import types
 if TYPE_CHECKING:
@@ -349,19 +349,6 @@ def build_command(executable: str, parameter: str) -> str:
     return command
 
 
-def get_file_path_from_dnd_dropped_uri(uri: str) -> str:
-    path = unquote(uri)  # escape special chars
-    path = path.strip('\r\n\x00')  # remove \r\n and NULL
-    # get the path to file
-    if re.match('^file:///[a-zA-Z]:/', path):  # windows
-        path = path[8:]  # 8 is len('file:///')
-    elif path.startswith('file://'):  # nautilus, rox
-        path = path[7:]  # 7 is len('file://')
-    elif path.startswith('file:'):  # xffm
-        path = path[5:]  # 5 is len('file:')
-    return path
-
-
 def sanitize_filename(filename: str) -> str:
     '''
     Sanitize filename of elements not allowed on Windows
@@ -779,22 +766,6 @@ def version_condition(current_version: str, required_version: str) -> bool:
     return True
 
 
-def get_available_emoticon_themes() -> list[str]:
-    files: list[Path] = []
-    for folder in configpaths.get('EMOTICONS').iterdir():
-        if not folder.is_dir():
-            continue
-        files += [theme for theme in folder.iterdir() if theme.is_file()]
-
-    my_emots = configpaths.get('MY_EMOTS')
-    if my_emots.is_dir():
-        files += list(my_emots.iterdir())
-
-    emoticons_themes = ['font']
-    emoticons_themes += [file.stem for file in files if file.suffix == '.png']
-    return sorted(emoticons_themes)
-
-
 def call_counter(func):
     def helper(self, restart=False):
         if restart:
@@ -936,74 +907,122 @@ def catch_exceptions(func):
     return func_wrapper
 
 
-def parse_uri_actions(uri: str) -> tuple[str, dict[str, str]]:
-    uri = uri[5:]
-    if '?' not in uri:
-        return 'message', {'jid': uri}
+def is_registered_uri_scheme(scheme: str) -> bool:
+    if scheme in iana.URI_SCHEMES:
+        return True
+    return scheme in app.settings.get('additional_uri_schemes').split()
 
-    jid, action = uri.split('?', 1)
-    data = {'jid': jid}
-    if ';' in action:
-        action, keys = action.split(';', 1)
-        action_keys = keys.split(';')
-        for key in action_keys:
-            if key.startswith('subject='):
-                data['subject'] = unquote(key[8:])
 
-            elif key.startswith('body='):
-                data['body'] = unquote(key[5:])
+def parse_xmpp_uri_query(pct_iquerycomp: str) -> tuple[str, dict[str, str]]:
+    '''
+    Parses 'mess%61ge;b%6Fdy=Hello%20%F0%9F%8C%90%EF%B8%8F' into
+    ('message', {'body': 'Hello 🌐️'}), empty string into ('', {}).
+    '''
+    # Syntax and terminology from
+    # <https://rfc-editor.org/rfc/rfc5122#section-2.2>; the pct_ prefix means
+    # percent encoding not being undone yet.
 
-            elif key.startswith('thread='):
-                data['thread'] = key[7:]
-    return action, data
+    if not pct_iquerycomp:
+        return '', {}
+
+    pct_iquerytype, _, pct_ipairs = pct_iquerycomp.partition(';')
+    iquerytype = unquote(pct_iquerytype, errors='strict')
+    if not pct_ipairs:
+        return iquerytype, {}
+
+    pairs: dict[str, str] = {}
+    for pct_ipair in pct_ipairs.split(';'):
+        pct_ikey, _, pct_ival = pct_ipair.partition('=')
+        pairs[
+            unquote(pct_ikey, errors='strict')
+        ] = unquote(pct_ival, errors='replace')
+    return iquerytype, pairs
 
 
 def parse_uri(uri: str) -> URI:
-    if uri.startswith('xmpp:'):
-        action, data = parse_uri_actions(uri)
-        try:
-            validate_jid(data['jid'])
-            return URI(type=URIType.XMPP,
-                       action=URIAction(action),
-                       data=data)
-        except ValueError:
-            # Unknown action
-            return URI(type=URIType.UNKNOWN)
-
-    if uri.startswith('mailto:'):
-        uri = uri[7:]
-        return URI(type=URIType.MAIL, data=uri)
-
-    if uri.startswith('tel:'):
-        uri = uri[4:]
-        return URI(type=URIType.TEL, data=uri)
-
-    if STH_AT_STH_DOT_STH_REGEX.match(uri):
-        return URI(type=URIType.AT, data=uri)
-
-    if uri.startswith('geo:'):
-        location = uri[4:]
-        lat, _, lon = location.partition(',')
-        if not lon:
-            return URI(type=URIType.UNKNOWN, data=uri)
-
-        if Gio.AppInfo.get_default_for_uri_scheme('geo'):
-            return URI(type=URIType.GEO, data=uri)
-
-        uri = geo_provider_from_location(lat, lon)
-        return URI(type=URIType.GEO, data=uri)
-
-    if uri.startswith('file://'):
-        return URI(type=URIType.FILE, data=uri)
-
     try:
         urlparts = urlparse(uri)
-        if not urlparts.scheme:
-            return URI(type=URIType.WEB, data=f'http://{uri}')
-    except Exception:
-        pass
+    except Exception as err:
+        return URI(URIType.INVALID, uri, data={'error': str(err)})
 
-    return URI(type=URIType.WEB, data=uri)
+    if not urlparts.scheme:
+        return URI(URIType.INVALID, uri, data={'error': 'Relative URI'})
+
+    scheme = urlparts.scheme  # urlparse is expected to return it in lower case
+
+    if not is_registered_uri_scheme(scheme):
+        return URI(URIType.INVALID, uri, data={'error': 'Unregistered scheme'})
+
+    if scheme in ('https', 'http'):
+        if not urlparts.netloc:
+            err = f'No host or empty host in an {scheme} URI'
+            return URI(URIType.INVALID, uri, data={'error': err})
+        return URI(URIType.WEB, uri, data=uri)
+
+    if scheme == 'xmpp':
+        if not urlparts.path.startswith('/'):
+            pct_jid = urlparts.path
+        else:
+            pct_jid = urlparts.path[1:]
+
+        data: dict[str, str] = {}
+        try:
+            data['jid'] = unquote(pct_jid, errors='strict')
+            validate_jid(data['jid'])
+            qtype, qparams = parse_xmpp_uri_query(urlparts.query)
+        except ValueError as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+
+        return URI(URIType.XMPP, uri,
+                   query_type=qtype,
+                   query_params=qparams,
+                   data=data)
+
+    if scheme == 'mailto':
+        try:
+            data = unquote(urlparts.path, errors='strict')
+            validate_jid(data, 'bare')  # meh, good enuf
+        except ValueError as err:
+            data = {'error': str(err)}
+            return URI(URIType.INVALID, uri, data=data)
+        return URI(URIType.MAIL, uri, data=data)
+
+    if scheme == 'tel':
+        data = uri[4:]
+        return URI(URIType.TEL, uri, data=data)
+
+    if scheme == 'geo':
+        # TODO: unify with .preview_helpers.split_geo_uri
+        # https://rfc-editor.org/rfc/rfc5870#section-3.3
+        lat, _, lon_alt = urlparts.path.partition(',')
+        if not lat or not lon_alt:
+            return URI(URIType.INVALID, uri, data={
+                       'error': 'No latitude or longitude'})
+        lon, _, _alt = lon_alt.partition(',')
+        if not lon:
+            return URI(URIType.INVALID, uri, data={'error': 'No longitude'})
+
+        # TODO: move redirection to open_uri, parsing needs to be unit-testable
+        if Gio.AppInfo.get_default_for_uri_scheme('geo'):
+            return URI(URIType.GEO, uri, data=uri)
+
+        data = geo_provider_from_location(lat, lon)
+        return URI(URIType.GEO, uri, data=data)
+
+    if scheme == 'file':
+        # https://rfc-editor.org/rfc/rfc8089.html
+        data: dict[str, str] = {}
+        try:
+            data['netloc'] = urlparts.netloc
+            data['path'] = Gio.File.new_for_uri(uri).get_path()
+            assert data['path']
+        except Exception as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+        return URI(URIType.FILE, uri, data=data)
+
+    return URI(URIType.WEB, uri, data=uri)
 
 
 @catch_exceptions
@@ -1012,64 +1031,101 @@ def open_uri(uri: Union[URI, str], account: Optional[str] = None) -> None:
         uri = parse_uri(uri)
 
     if uri.type == URIType.FILE:
-        open_file(uri.data)
-
-    elif uri.type == URIType.TEL:
-        if sys.platform == 'win32':
-            webbrowser.open(f'tel:{uri.data}')
+        opt_name = 'allow_open_file_uris'
+        if app.settings.get(opt_name):
+            open_file_uri(uri.source)
         else:
-            Gio.AppInfo.launch_default_for_uri(f'tel:{uri.data}')
+            log.info('Blocked opening a file URI, see %s option', opt_name)
 
-    elif uri.type == URIType.MAIL:
-        if sys.platform == 'win32':
-            webbrowser.open(f'mailto:{uri.data}')
-        else:
-            Gio.AppInfo.launch_default_for_uri(f'mailto:{uri.data}')
+    elif uri.type in (URIType.MAIL, URIType.TEL):
+        open_uri_externally(uri.source)
 
     elif uri.type in (URIType.WEB, URIType.GEO):
-        if sys.platform == 'win32':
-            webbrowser.open(uri.data)
-        else:
-            Gio.AppInfo.launch_default_for_uri(uri.data)
+        open_uri_externally(uri.data)
 
-    elif uri.type == URIType.AT:
-        app.window.start_chat_from_jid(account, uri.data)
-
-    elif uri.type == URIType.XMPP:
+    elif uri.type in (URIType.XMPP, URIType.AT):
         if account is None:
             log.warning('Account must be specified to open XMPP uri')
             return
 
-        if isinstance(uri.data, dict):
+        if uri.type == URIType.XMPP:
             jid = uri.data['jid']
-            message = uri.data.get('body')
         else:
-            log.warning('Cant open URI: %s', uri)
-            return
+            jid = uri.data
 
-        if uri.action == URIAction.JOIN:
+        qtype, qparams = XmppUriQuery.from_str(uri.query_type), uri.query_params
+        if not qtype:
+            log.info('open_uri: can\'t "%s": '
+                     'unsupported query type in %s', uri.query_type, uri)
+            # From <rfc5122#section-2.5>:
+            # > If the processing application does not understand [...] the
+            # > specified query type, it MUST ignore the query component and
+            # > treat the IRI/URI as consisting of, for example,
+            # > <xmpp:example-node@example.com> rather than
+            # > <xmpp:example-node@example.com?query>."
+            qtype, qparams = XmppUriQuery.NONE, {}
+
+        if qtype == XmppUriQuery.JOIN:
             app.app.activate_action(
                 'groupchat-join',
                 GLib.Variant('as', [account, jid]))
-        elif uri.action == URIAction.MESSAGE:
-            app.window.start_chat_from_jid(account, jid, message=message)
         else:
-            log.warning('Cant open URI: %s', uri)
+            message = qparams.get('body')
+            app.window.start_chat_from_jid(account, jid, message=message)
+
+    elif uri.type == URIType.INVALID:
+        log.warning('open_uri: Invalid %s', uri)
+        # TODO: UI error instead
 
     else:
-        log.warning('Cant open URI: %s', uri)
+        log.error('open_uri: No handler for %s', uri)
+        # TODO: this is a bug, so, `raise` maybe?
+
+
+def open_uri_externally(uri: str) -> None:
+    if sys.platform == 'win32':
+        webbrowser.open(uri, new=2)
+    else:
+        try:
+            Gio.AppInfo.launch_default_for_uri(uri)
+        except GLib.Error as err:
+            log.info('open_uri_externally: '
+                     "Couldn't launch default for %s: %s", uri, err)
+
+
+def open_file_uri(uri: str) -> None:
+    try:
+        if sys.platform != 'win32':
+            Gio.AppInfo.launch_default_for_uri(uri)
+        else:
+            os.startfile(uri)
+    except Exception as err:
+        log.info("Couldn't open file URI %s: %s", uri, err)
 
 
 @catch_exceptions
-def open_file(path: Union[str, Path]) -> None:
-    if os.name == 'nt':
-        os.startfile(path)
-    else:
-        # Call str() to make it work with pathlib.Path
-        path = str(path)
-        if not path.startswith('file://'):
-            path = 'file://' + path
-        Gio.AppInfo.launch_default_for_uri(path)
+def open_file(path: Path) -> None:
+    if not path.exists():
+        log.warning('Unable to open file, path %s does not exist', path)
+        return
+
+    open_file_uri(path.as_uri())
+
+
+def filesystem_path_from_uri(uri: str) -> Optional[Path]:
+    puri = parse_uri(uri)
+    if puri.type != URIType.FILE:
+        return None
+
+    netloc = puri.data['netloc']
+    if netloc and netloc.lower() != 'localhost' and sys.platform != 'win32':
+        return None
+    return Path(puri.data['path'])
+
+
+def get_file_path_from_dnd_dropped_uri(text: str) -> Optional[Path]:
+    uri = text.strip('\r\n\x00')  # remove \r\n and NULL
+    return filesystem_path_from_uri(uri)
 
 
 def file_is_locked(path_to_file: str) -> bool:
@@ -1221,11 +1277,9 @@ def is_retraction_allowed(self_contact: types.GroupchatParticipant,
     return self_contact.affiliation >= contact.affiliation
 
 
-def get_tls_error_phrase(tls_error: Gio.TlsCertificateFlags) -> str:
-    phrase = GIO_TLS_ERRORS.get(tls_error)
-    if phrase is None:
-        return GIO_TLS_ERRORS[Gio.TlsCertificateFlags.GENERIC_ERROR]
-    return phrase
+def get_tls_error_phrases(tls_errors: set[Gio.TlsCertificateFlags]
+                          ) -> list[str]:
+    return [GIO_TLS_ERRORS[err] for err in tls_errors]
 
 
 class Observable:
@@ -1367,8 +1421,10 @@ def load_file_async(path: Path,
     file.load_contents_async(None, _on_load_finished)
 
 
-def convert_gio_to_openssl_cert(cert: Gio.TlsCertificate) -> X509:
-    return load_certificate(FILETYPE_PEM, cert.props.certificate_pem.encode())
+def get_x509_cert_from_gio_cert(cert: Gio.TlsCertificate) -> x509.Certificate:
+    glib_bytes = GLib.ByteArray.free_to_bytes(cert.props.certificate)
+    return x509.load_der_x509_certificate(
+        glib_bytes.get_data(), default_backend())
 
 
 def get_custom_host(account: str) -> Optional[tuple[str,
