@@ -20,10 +20,12 @@ import os
 from pathlib import Path
 from urllib.parse import urlparse
 from urllib.parse import ParseResult
+import uuid
 
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Soup
+from nbxmpp.util import convert_tls_error_flags
 
 from gajim.common import app
 from gajim.common import configpaths
@@ -31,7 +33,7 @@ from gajim.common.const import MIME_TYPES
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import load_file_async
 from gajim.common.helpers import write_file_async
-from gajim.common.helpers import get_tls_error_phrase
+from gajim.common.helpers import get_tls_error_phrases
 from gajim.common.helpers import get_account_proxy
 from gajim.common.i18n import _
 from gajim.common.preview_helpers import aes_decrypt
@@ -64,6 +66,8 @@ class Preview:
                  from_us: bool = False,
                  context: Optional[str] = None
                  ) -> None:
+
+        self.id = str(uuid.uuid4())
         self._uri = uri
         self._urlparts = urlparts
         self._filename = filename_from_uri(uri)
@@ -194,12 +198,20 @@ class PreviewManager:
         self._orig_dir = Path(configpaths.get('MY_DATA')) / 'downloads'
         self._thumb_dir = Path(configpaths.get('MY_CACHE')) / 'downloads.thumb'
 
+        self._previews: dict[str, Preview] = {}
+
         if GLib.mkdir_with_parents(str(self._orig_dir), 0o700) != 0:
             log.error('Failed to create: %s', self._orig_dir)
 
         if GLib.mkdir_with_parents(str(self._thumb_dir), 0o700) != 0:
             log.error('Failed to create: %s', self._thumb_dir)
 
+    def get_preview(self, preview_id: str) -> Optional[Preview]:
+        return self._previews.get(preview_id)
+
+    def clear_previews(self) -> None:
+        self._previews.clear()
+
     def _get_session(self, account: str) -> Soup.Session:
         if account not in self._sessions:
             self._sessions[account] = self._create_session(account)
@@ -282,12 +294,11 @@ class PreviewManager:
         if not self._accept_uri(urlparts, uri, additional_data):
             return False
 
-        if uri.startswith('geo:'):
+        if urlparts.scheme == 'geo':
             try:
                 split_geo_uri(uri)
             except Exception as err:
-                log.error(uri)
-                log.error(err)
+                log.info('Bad geo URI %s: %s', uri, err)
                 return False
 
         return True
@@ -301,9 +312,11 @@ class PreviewManager:
         if uri.startswith('geo:'):
             preview = Preview(uri, None, None, None, 96, widget)
             preview.update_widget()
+            self._previews[preview.id] = preview
             return
 
         preview = self._process_web_uri(uri, widget, from_us, context)
+        self._previews[preview.id] = preview
 
         if not preview.orig_exists:
             if context is not None and not from_us:
@@ -430,11 +443,12 @@ class PreviewManager:
             return
 
         if tls_errors:
-            phrase = get_tls_error_phrase(tls_errors)
-            log.warning('TLS verification failed: %s', phrase)
+            phrases = get_tls_error_phrases(convert_tls_error_flags(tls_errors))
+            log.warning(
+                'TLS verification failed: %s (0x%02x)', phrases, tls_errors)
             session = self._get_session(preview.account)
             session.cancel_message(message, Soup.Status.CANCELLED)
-            preview.info_message = _('TLS verification failed: %s') % phrase
+            preview.info_message = _('TLS verification failed: %s') % phrases[0]
             preview.update_widget()
 
     def _on_content_sniffed(self,
