@@ -121,13 +121,18 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self.connect('destroy', self._on_destroy)
 
         # pylint: disable=line-too-long
-        connect_destroy(self._ui.tree_selection, 'changed', self._on_group_selection_changed)
-        connect_destroy(self._ui.toggle_renderer, 'toggled', self._on_group_toggled)
-        connect_destroy(self._ui.text_renderer, 'edited', self._on_group_name_edited)
+        connect_destroy(self._ui.tree_selection,
+                        'changed', self._on_group_selection_changed)
+        connect_destroy(self._ui.toggle_renderer,
+                        'toggled', self._on_group_toggled)
+        connect_destroy(self._ui.text_renderer,
+                        'edited', self._on_group_name_edited)
 
         self.register_events([
-            ('subscribed-presence-received', ged.GUI1, self._on_subscribed_presence_received),
-            ('unsubscribed-presence-received', ged.GUI1, self._on_unsubscribed_presence_received),
+            ('subscribed-presence-received', ged.GUI1,
+             self._on_subscribed_presence_received),
+            ('unsubscribed-presence-received', ged.GUI1,
+             self._on_unsubscribed_presence_received),
         ])
         # pylint: enable=line-too-long
 
@@ -327,10 +332,19 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self._ui.name_entry.set_sensitive(active)
         if active:
             self._ui.name_entry.grab_focus()
+            self._ui.edit_name_button_image.set_from_icon_name(
+                'document-save-symbolic', Gtk.IconSize.BUTTON)
+        else:
+            self._ui.edit_name_button_image.set_from_icon_name(
+                'document-edit-symbolic', Gtk.IconSize.BUTTON)
+            name = self._ui.name_entry.get_text()
+            if name == self.contact.name:
+                return
 
-        name = self._ui.name_entry.get_text()
-        self._client.get_module('Roster').set_item(self.contact.jid, name)
-        self._ui.contact_name_label.set_text(name)
+            assert isinstance(self.contact, BareContact)
+            self._client.get_module('Roster').set_item(
+                self.contact.jid, name, self.contact.groups)
+            self._ui.contact_name_label.set_text(name)
 
     def _on_name_entry_activate(self, _widget: Gtk.Entry) -> None:
         self._ui.edit_name_button.set_active(False)
