@@ -29,6 +29,7 @@ from enum import IntEnum
 from enum import unique
 from datetime import datetime
 from datetime import timezone
+from pathlib import Path
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -59,7 +60,6 @@ from .dialogs import DialogButton
 from .dialogs import ConfirmationDialog
 from .dialogs import ErrorDialog
 from .filechoosers import FileSaveDialog
-from .file_transfer_send import SendFileDialog
 from .tooltips import FileTransfersTooltip
 from .builder import get_builder
 from .util import format_eta
@@ -404,16 +404,9 @@ class FileTransfersWindow:
                                text=_('_Download Again'),
                                callback=_on_yes)]).show()
 
-    def show_file_send_request(self,
-                               account: str,
-                               contact: types.BareContact
-                               ) -> None:
-        send_callback = partial(self.send_file, account, contact)
-        SendFileDialog(contact, send_callback, self.window)
-
     def send_file(self,
                   account: str,
-                  contact: types.BareContact,
+                  contact: types.ResourceContact,
                   resource_jid: JID,
                   file_path: str,
                   file_desc: str = ''
@@ -1030,9 +1023,9 @@ class FileTransfersWindow:
         file_props = FilesProp.getFilePropByType(sid[0], sid[1:])
         if not file_props.file_name:
             return
-        path = os.path.split(file_props.file_name)[0]
-        if os.path.exists(path) and os.path.isdir(path):
-            open_file(path)
+
+        folder = Path(file_props.file_name).parent
+        open_file(folder)
 
     def _on_cancel_menuitem_activate(self, widget):
         self._on_cancel_button_clicked(widget)
