@@ -18,14 +18,12 @@ from typing import Any
 from typing import Optional
 
 import os
-import sys
 import tempfile
 import logging
 import uuid
 
 from gi.repository import GLib
 from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import Gtk
 from gi.repository import GObject
@@ -45,10 +43,7 @@ from gajim.common.types import ChatContactT
 from gajim.gui.security_label_selector import SecurityLabelSelector
 
 from .builder import get_builder
-from .dialogs import DialogButton
 from .dialogs import ErrorDialog
-from .dialogs import PastePreviewDialog
-from .emoji_chooser import emoji_chooser
 from .menus import get_encryption_menu
 from .menus import get_format_menu
 from .menus import get_groupchat_menu
@@ -68,7 +63,7 @@ class MessageActionsBox(Gtk.Grid):
         self._contact: Optional[ChatContactT] = None
 
         self._ui = get_builder('message_actions_box.ui')
-        self.get_style_context().add_class('message-action-box')
+        self.get_style_context().add_class('message-actions-box')
 
         self.attach(self._ui.box, 0, 0, 1, 1)
 
@@ -101,8 +96,6 @@ class MessageActionsBox(Gtk.Grid):
 
         self._ui.quick_invite_button.set_action_name('win.muc-invite')
 
-        self._init_emoticon_popover()
-
         self.show_all()
         self._ui.connect_signals(self)
 
@@ -160,7 +153,7 @@ class MessageActionsBox(Gtk.Grid):
             self._on_format(action_name)
 
         elif action_name == 'show-emoji-chooser':
-            self._on_show_emoji_chooser()
+            self.msg_textview.emit('insert-emoji')
 
         elif action_name == 'quote':
             assert param
@@ -392,31 +385,6 @@ class MessageActionsBox(Gtk.Grid):
             menu = get_singlechat_menu(contact)
         self._ui.settings_menu.set_menu_model(menu)
 
-    def _init_emoticon_popover(self) -> None:
-        if not app.settings.get('emoticons_theme'):
-            return
-
-        if sys.platform == 'darwin':
-            emoji_chooser.text_widget = self.msg_textview
-            self._ui.emoticons_button.set_popover(emoji_chooser)
-            return
-
-        self._ui.emoticons_button.set_sensitive(True)
-        self._ui.emoticons_button.connect('clicked',
-                                          self._on_emoticon_button_clicked)
-
-    def toggle_emoticons(self) -> None:
-        if app.settings.get('emoticons_theme'):
-            self._ui.emoticons_button.set_no_show_all(False)
-            self._ui.emoticons_button.show()
-        else:
-            self._ui.emoticons_button.set_no_show_all(True)
-            self._ui.emoticons_button.hide()
-
-    def _on_emoticon_button_clicked(self, _widget: Gtk.Button) -> None:
-        self.msg_textview.emit('insert-emoji')
-        self._ui.emoticons_button.set_active(False)
-
     def _on_format(self, name: str) -> None:
         name = name.removeprefix('input-')
         self.msg_textview.apply_formatting(name)
@@ -424,14 +392,6 @@ class MessageActionsBox(Gtk.Grid):
     def _on_clear(self) -> None:
         self.msg_textview.clear()
 
-    def _on_show_emoji_chooser(self) -> None:
-        if sys.platform == 'darwin':
-            popover = self._ui.emoticons_button.get_popover()
-            assert popover
-            popover.show()
-        else:
-            self.msg_textview.emit('insert-emoji')
-
     def _on_send_file_enabled_changed(self,
                                       action: Gio.SimpleAction,
                                       _param: GObject.ParamSpec) -> None:
@@ -577,32 +537,16 @@ class MessageActionsBox(Gtk.Grid):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
         image = clipboard.wait_for_image()
         if image is None:
+            log.info('No image pasted')
             return
 
-        PastePreviewDialog(
-            _('Paste Image'),
-            _('You are trying to paste an image'),
-            _('Are you sure you want to paste your '
-              "clipboard's image into the chat window?"),
-            _('_Do not ask me again'),
-            image,
-            [DialogButton.make('Cancel'),
-             DialogButton.make('Accept',
-                               text=_('_Paste'),
-                               callback=self._paste_event_confirmed,
-                               args=[image])]).show()
+        temp_dir = tempfile.gettempdir()
+        image_path = os.path.join(temp_dir, f'{uuid.uuid4()}.png')
 
-    def _paste_event_confirmed(self,
-                               image: GdkPixbuf.Pixbuf
-                               ) -> None:
-
-        dir_ = tempfile.gettempdir()
-        path = os.path.join(dir_, f'{uuid.uuid4()}.png')
+        image.savev(image_path, 'png', [], [])
         if image is None:
             log.error('Could not process pasted image')
             return
 
-        image.savev(path, 'png', [], [])
-
-        assert self._contact is not None
-        app.interface.start_file_transfer(self._contact, path)
+        app.window.activate_action(
+            'send-file', GLib.Variant('as', [image_path]))
