@@ -1,5 +1,10 @@
 import unittest
+from unittest.mock import MagicMock
 
+from gajim import gui
+gui.init('gtk')
+
+from gajim.common import app  # Avoids circular imports from common.helpers
 import gajim.common.styling as styling
 from gajim.common.styling import PlainBlock
 from gajim.common.styling import PreBlock
@@ -13,6 +18,10 @@ from gajim.common.styling import process_uris
 from gajim.common.text_helpers import jid_to_iri
 
 
+app.settings = MagicMock()
+app.settings.get.return_value = 'a a- a. scheme'  # additional_uri_schemes
+
+
 STYLING = {
     'pre cannot have children':  {
         'input': '_no pre `with *children*`_',
@@ -287,7 +296,7 @@ URIS = [
     'a:b',
     'a-:b',
     'a.:b',
-    'xmpp:conference.gajim.org'
+    'xmpp:conference.gajim.org',
     'xmpp:asd@at',
     'xmpp:asd@asd.at',
     'xmpp:asd-asd@asd.asdasd.at.',
@@ -297,6 +306,13 @@ URIS = [
     'xmpp:%23room%25irc.example@biboumi.xmpp.example',
     'xmpp:+15551234567@cheogram.com',
     'xmpp:romeo@montague.net?message;subject=Test%20Message;body=Here%27s%20a%20test%20message',
+    'geo:1,2',
+    'geo:1,2,3',
+    'file:/foo/bar/baz',  # xffm
+    'file:///foo/bar/baz',  # nautilus, rox
+    'file:///x:/foo/bar/baz',  # windows
+    'file://localhost/foo/bar/baz',
+    'file://nonlocalhost/foo/bar/baz',
 
     # These seem to be from https://mathiasbynens.be/demo/url-regex
     'http://foo.com/blah_blah',
@@ -373,6 +389,14 @@ NONURIS = [
     ' ',
     '.',
     ':',
+    '://',
+    'Böblingen:🌥',
+    'Bo\u0308blingen:🌥',  # finds blingen:🌥 but rejects an unregistered scheme
+    'path/to/file_na.me:123',
+    '_sche.me:body_',
+    '_sche+me:body_',
+    '.scheme:body',
+    '+scheme:body',
 
     # These are from https://mathiasbynens.be/demo/url-regex
     '//',
@@ -380,10 +404,6 @@ NONURIS = [
     '///a',
     '///',
     'foo.com',
-    'http://foo.bar?q=Spaces should be encoded',
-    'http:// shouldfail.com',
-    ':// should fail',
-    'http://foo.bar/foo(bar)baz quux',
 ]
 
 
@@ -401,6 +421,23 @@ UNACCEPTABLE_URIS = [
     'http://##',
     'http://##/',
     'http:///a',
+
+    'geo:1,',
+    'geo:,2',
+    #'geo:1,2,',  FIXME: wrongly parsed as valid
+    'geo:1,,3',
+    'geo:,2,3',
+    'geo:1,,',
+    'geo:,2,',
+    'geo:,,3',
+    'geo:,,',
+
+    'file:',
+    'file:a',
+    'file:a/',
+    'file:a/b',
+
+    'mailtomailto:foo@bar.com.uk',
 ]
 
 
@@ -458,7 +495,6 @@ NONJIDS = [
 
     # These are from https://rfc-editor.org/rfc/rfc7622#section-3.5
     '"juliet"@example.com',
-    'foo bar@example.com',  # search is expected to find 'bar@example.com'
     '@example.com',
     'henryⅣ@example.com',  # localpart has a compatibility-decomposable cp
     '♚@example.com',        # localpart has a symbol cp
@@ -469,13 +505,14 @@ NONJIDS = [
 URIS_WITH_TEXT = [
     ('write to my email mailto:foo@bar.com.uk (but not to mailto:bar@foo.com)',
      ['mailto:foo@bar.com.uk', 'mailto:bar@foo.com']),
-    ('write to my email mailtomailto:foo@bar.com.uk (but not to mailto:bar@foo.com)',
-     ['mailtomailto:foo@bar.com.uk', 'mailto:bar@foo.com']),
     ('see this http://userid@example.com/ link', ['http://userid@example.com/']),
     ('see this http://userid@example.com/, and ..', ['http://userid@example.com/']),
     ('<http://userid@example.com/>', ['http://userid@example.com/']),
     ('"http://userid@example.com/"', ['http://userid@example.com/']),
     ('regexes are useless (see https://en.wikipedia.org/wiki/Recursion_(computer_science)), but comfy', ['https://en.wikipedia.org/wiki/Recursion_(computer_science)']),
+    ('(scheme:body)', ['scheme:body']),
+    ('/scheme:body', ['scheme:body']),
+    ('!scheme:body', ['scheme:body']),
 ]
 
 
@@ -493,46 +530,32 @@ class Test(unittest.TestCase):
         for uri in URIS:
             text = self.wrap(uri)
             hlinks = process_uris(text)
-            self.assertEqual(len(hlinks), 1, text)
-            self.assertEqual(hlinks[0].uri, uri, text)
+            self.assertEqual([l.uri for l in hlinks], [uri], text)
 
     def test_invalid_uris(self):
         for foo in NONURIS + UNACCEPTABLE_URIS:
-            if foo == 'scheme:' or\
-               foo == 'http://':
-                continue  # these currently fial (FIXME)
             text = self.wrap(foo)
             hlinks = process_uris(text)
-            if len(hlinks) == 0:
-                continue
-            self.assertEqual(len(hlinks), 1, text)
-            self.assertNotEqual(hlinks[0].text, foo, text)
+            self.assertEqual([l.text for l in hlinks], [], text)
 
     def test_jids(self):
         for jid in JIDS:
             text = self.wrap(jid)
+            uri = jid_to_iri(jid)
             hlinks = process_uris(text)
-            self.assertEqual(len(hlinks), 1, text)
-            self.assertEqual(hlinks[0].text, jid, text)
-            self.assertEqual(hlinks[0].uri, jid_to_iri(jid), text)
+            self.assertEqual([(l.text, l.uri) for l in hlinks],
+                             [(jid, uri)], text)
 
     def test_nonjids(self):
         for foo in NONJIDS:
-            if foo == 'juliet@':
-                continue  # this currently fials (FIXME)
             text = self.wrap(foo)
             hlinks = process_uris(text)
-            if len(hlinks) == 0:
-                continue
-            self.assertEqual(len(hlinks), 1, text)
-            self.assertNotEqual(hlinks[0].text, foo, text)
+            self.assertEqual([l.text for l in hlinks], [], text)
 
     def test_uris_with_text(self):
-        for text, result in URIS_WITH_TEXT:
+        for text, results in URIS_WITH_TEXT:
             hlinks = process_uris(text)
-            self.assertEqual(len(hlinks), len(result), text)
-            for i, res in enumerate(result):
-                self.assertEqual(hlinks[i].text, res, text)
+            self.assertEqual([l.text for l in hlinks], results, text)
 
 
 if __name__ == "__main__":
